@@ -132,7 +132,7 @@ for (my $i = 0; $i < scalar(@services); $i++) {
 }
 print "};\n";
 print "buffer_t config_all_services_buf = {\n";
-print "\tconfig_all_services, sizeof(config_all_services), { 0, }\n";
+print "\tconfig_all_services, sizeof(config_all_services), { NULL, }\n";
 print "};\n";
 
 print "const struct setting_parser_info *all_default_roots[] = {\n";
