@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -36,11 +36,18 @@ struct director_request {
 	time_t create_time;
 	unsigned int username_hash;
 	enum director_request_delay_reason delay_reason;
+	char *username_tag;
 
 	director_request_callback *callback;
 	void *context;
 };
 
+static void director_request_free(struct director_request *request)
+{
+	i_free(request->username_tag);
+	i_free(request);
+}
+
 static const char *
 director_request_get_timeout_error(struct director_request *request,
 				   struct user *user, string_t *str)
@@ -68,7 +75,10 @@ director_request_get_timeout_error(struct director_request *request,
 		str_printfa(str, ", user refreshed %u secs ago",
 			    (unsigned int)(ioloop_time - user->timestamp));
 	}
-	str_printfa(str, "hash=%u)", request->username_hash);
+	str_printfa(str, ", hash=%u", request->username_hash);
+	if (request->username_tag != NULL)
+		str_printfa(str, ", tag=%s", request->username_tag);
+	str_append_c(str, ')');
 	return str_c(str);
 }
 
@@ -103,7 +113,7 @@ static void director_request_timeout(struct director *dir)
 		T_BEGIN {
 			request->callback(NULL, errormsg, request->context);
 		} T_END;
-		i_free(request);
+		director_request_free(request);
 	}
 
 	if (array_count(&dir->pending_requests) == 0 && dir->to_request != NULL)
@@ -111,6 +121,7 @@ static void director_request_timeout(struct director *dir)
 }
 
 void director_request(struct director *dir, const char *username,
+		      const char *tag,
 		      director_request_callback *callback, void *context)
 {
 	struct director_request *request;
@@ -121,6 +132,7 @@ void director_request(struct director *dir, const char *username,
 	request->dir = dir;
 	request->create_time = ioloop_time;
 	request->username_hash = username_hash;
+	request->username_tag = tag[0] == '\0' ? NULL : i_strdup(tag);
 	request->callback = callback;
 	request->context = context;
 
@@ -159,7 +171,8 @@ static void ring_log_delayed_warning(struct director *dir)
 }
 
 static bool
-director_request_existing(struct director_request *request, struct user *user)
+director_request_existing(struct director_request *request, struct user *user,
+			  const char *tag)
 {
 	struct director *dir = request->dir;
 	struct mail_host *host;
@@ -193,7 +206,7 @@ director_request_existing(struct director_request *request, struct user *user)
 
 	/* user is close to being expired. another director may have
 	   already expired it. */
-	host = mail_host_get_by_hash(dir->mail_hosts, user->username_hash);
+	host = mail_host_get_by_hash(dir->mail_hosts, user->username_hash, tag);
 	if (!dir->ring_synced) {
 		/* try again later once ring is synced */
 		request->delay_reason = REQUEST_DELAY_RINGNOTSYNCED;
@@ -253,6 +266,7 @@ bool director_request_continue(struct director_request *request)
 	struct director *dir = request->dir;
 	struct mail_host *host;
 	struct user *user;
+	const char *tag;
 
 	if (!dir->ring_handshaked) {
 		/* delay requests until ring handshaking is complete */
@@ -264,8 +278,9 @@ bool director_request_continue(struct director_request *request)
 	}
 
 	user = user_directory_lookup(dir->users, request->username_hash);
+	tag = request->username_tag == NULL ? "" : request->username_tag;
 	if (user != NULL) {
-		if (!director_request_existing(request, user))
+		if (!director_request_existing(request, user, tag))
 			return FALSE;
 		user_directory_refresh(dir->users, user);
 		dir_debug("request: %u refreshed timeout to %u",
@@ -280,7 +295,7 @@ bool director_request_continue(struct director_request *request)
 			return FALSE;
 		}
 		host = mail_host_get_by_hash(dir->mail_hosts,
-					     request->username_hash);
+					     request->username_hash, tag);
 		if (host == NULL) {
 			/* all hosts have been removed */
 			request->delay_reason = REQUEST_DELAY_NOHOSTS;
@@ -299,6 +314,6 @@ bool director_request_continue(struct director_request *request)
 	T_BEGIN {
 		request->callback(&user->host->ip, NULL, request->context);
 	} T_END;
-	i_free(request);
+	director_request_free(request);
 	return TRUE;
 }
