@@ -6,8 +6,10 @@ struct director_settings {
 
 	const char *director_servers;
 	const char *director_mail_servers;
+	const char *director_username_hash;
 	unsigned int director_user_expire;
 	unsigned int director_doveadm_port;
+	bool director_consistent_hashing;
 };
 
 extern const struct setting_parser_info director_setting_parser_info;
