@@ -1,9 +1,10 @@
-/* Copyright (c) 2010-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
 #include "array.h"
 #include "str.h"
+#include "strescape.h"
 #include "ipc-client.h"
 #include "user-directory.h"
 #include "mail-host.h"
@@ -274,7 +275,9 @@ void director_set_ring_synced(struct director *dir)
 	if (dir->to_handshake_warning != NULL)
 		timeout_remove(&dir->to_handshake_warning);
 	if (dir->ring_handshake_warning_sent) {
-		i_warning("Ring is synced, continuing delayed requests");
+		i_warning("Ring is synced, continuing delayed requests "
+			  "(syncing took %d secs)",
+			  (int)(ioloop_time - dir->ring_last_sync_time));
 		dir->ring_handshake_warning_sent = FALSE;
 	}
 
@@ -501,6 +504,8 @@ void director_update_host(struct director *dir, struct director_host *src,
 			  struct director_host *orig_src,
 			  struct mail_host *host)
 {
+	string_t *str;
+
 	/* update state in case this is the first mail host being added */
 	director_set_state_changed(dir);
 
@@ -509,10 +514,25 @@ void director_update_host(struct director *dir, struct director_host *src,
 		orig_src->last_seq++;
 	}
 
-	director_update_send(dir, src, t_strdup_printf(
-		"HOST\t%s\t%u\t%u\t%s\t%u\n",
-		net_ip2addr(&orig_src->ip), orig_src->port, orig_src->last_seq,
-		net_ip2addr(&host->ip), host->vhost_count));
+	str = t_str_new(128);
+	str_printfa(str, "HOST\t%s\t%u\t%u\t%s\t%u",
+		    net_ip2addr(&orig_src->ip), orig_src->port,
+		    orig_src->last_seq,
+		    net_ip2addr(&host->ip), host->vhost_count);
+	if (host->tag[0] == '\0')
+		;
+	else if (dir->ring_handshaked &&
+		 dir->ring_min_version < DIRECTOR_VERSION_TAGS) {
+		i_error("Ring has directors that don't support tags - removing host %s with tag '%s'",
+			net_ip2addr(&host->ip), host->tag);
+		director_remove_host(dir, NULL, NULL, host);
+		return;
+	} else {
+		str_append_c(str, '\t');
+		str_append_tabescaped(str, host->tag);
+	}
+	str_append_c(str, '\n');
+	director_update_send(dir, src, str_c(str));
 	director_sync(dir);
 }
 
@@ -907,7 +927,7 @@ director_init(const struct director_settings *set,
 	i_array_init(&dir->connections, 8);
 	dir->users = user_directory_init(set->director_user_expire,
 					 set->director_username_hash);
-	dir->mail_hosts = mail_hosts_init();
+	dir->mail_hosts = mail_hosts_init(set->director_consistent_hashing);
 
 	dir->ipc_proxy = ipc_client_init(DIRECTOR_IPC_PROXY_PATH);
 	dir->ring_min_version = DIRECTOR_VERSION_MINOR;
