@@ -1,4 +1,4 @@
-/* Copyright (c) 2013-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2013-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -14,15 +14,22 @@
 #include "dsync-brain-private.h"
 
 static int
-ns_mailbox_try_alloc(struct mail_namespace *ns, const guid_128_t guid,
-		     struct mailbox **box_r, const char **error_r)
+ns_mailbox_try_alloc(struct dsync_brain *brain, struct mail_namespace *ns,
+		     const guid_128_t guid, struct mailbox **box_r,
+		     const char **error_r)
 {
+	enum mailbox_flags flags = 0;
 	struct mailbox *box;
 	enum mailbox_existence existence;
 	enum mail_error err;
 	int ret;
 
-	box = mailbox_alloc_guid(ns->list, guid, 0);
+	if (brain->backup_send) {
+		/* make sure mailbox isn't modified */
+		flags |= MAILBOX_FLAG_READONLY;
+	}
+
+	box = mailbox_alloc_guid(ns->list, guid, flags);
 	ret = mailbox_exists(box, FALSE, &existence);
 	if (ret < 0) {
 		*error_r = mailbox_get_last_error(box, &err);
@@ -51,7 +58,7 @@ int dsync_brain_mailbox_alloc(struct dsync_brain *brain, const guid_128_t guid,
 	for (ns = brain->user->namespaces; ns != NULL; ns = ns->next) {
 		if (!dsync_brain_want_namespace(brain, ns))
 			continue;
-		if ((ret = ns_mailbox_try_alloc(ns, guid, box_r, error_r)) != 0) {
+		if ((ret = ns_mailbox_try_alloc(brain, ns, guid, box_r, error_r)) != 0) {
 			if (ret < 0)
 				brain->failed = TRUE;
 			return ret;
@@ -213,13 +220,16 @@ dsync_brain_sync_mailbox_init_remote(struct dsync_brain *brain,
 		import_flags |= DSYNC_MAILBOX_IMPORT_FLAG_MAILS_USE_GUID128;
 
 	brain->box_importer = brain->backup_send ? NULL :
-		dsync_mailbox_import_init(brain->box, brain->log_scan,
+		dsync_mailbox_import_init(brain->box, brain->virtual_all_box,
+					  brain->log_scan,
 					  last_common_uid, last_common_modseq,
 					  last_common_pvt_modseq,
 					  remote_dsync_box->uid_next,
 					  remote_dsync_box->first_recent_uid,
 					  remote_dsync_box->highest_modseq,
 					  remote_dsync_box->highest_pvt_modseq,
+					  brain->sync_since_timestamp,
+					  brain->sync_flag,
 					  import_flags);
 }
 
@@ -306,6 +316,8 @@ int dsync_brain_sync_mailbox_open(struct dsync_brain *brain,
 		exporter_flags |= DSYNC_MAILBOX_EXPORTER_FLAG_MAILS_HAVE_GUIDS;
 	if (brain->no_mail_prefetch)
 		exporter_flags |= DSYNC_MAILBOX_EXPORTER_FLAG_MINIMAL_DMAIL_FILL;
+	if (brain->sync_since_timestamp > 0)
+		exporter_flags |= DSYNC_MAILBOX_EXPORTER_FLAG_TIMESTAMPS;
 
 	brain->box_exporter = brain->backup_recv ? NULL :
 		dsync_mailbox_export_init(brain->box, brain->log_scan,
