@@ -52,9 +52,12 @@ struct dsync_brain {
 	const char *process_title_prefix;
 	ARRAY(struct mail_namespace *) sync_namespaces;
 	const char *sync_box;
+	struct mailbox *virtual_all_box;
 	guid_128_t sync_box_guid;
 	const char *const *exclude_mailboxes;
 	enum dsync_brain_sync_type sync_type;
+	time_t sync_since_timestamp;
+	const char *sync_flag;
 	char alt_char;
 
 	unsigned int lock_timeout;
