@@ -47,6 +47,10 @@ struct dsync_brain_settings {
 	ARRAY(struct mail_namespace *) sync_namespaces;
 	/* Sync only this mailbox name */
 	const char *sync_box;
+	/* Use this virtual \All mailbox to be able to copy mails with the same
+	   GUID instead of saving them twice. With most storages this results
+	   in less disk space usage. */
+	const char *virtual_all_box;
 	/* Sync only this mailbox GUID */
 	guid_128_t sync_box_guid;
 	/* Exclude these mailboxes from the sync. They can contain '*'
@@ -55,6 +59,11 @@ struct dsync_brain_settings {
 	/* Alternative character to use in mailbox names where the original
 	   character cannot be used. */
 	char mailbox_alt_char;
+	/* Sync only mails with received timestamp at least this high. */
+	time_t sync_since_timestamp;
+	/* Sync only mails which contains / doesn't contain this flag.
+	   '-' at the beginning means this flag must not exist. */
+	const char *sync_flag;
 
 	/* If non-zero, use dsync lock file for this user */
 	unsigned int lock_timeout_secs;
