@@ -1,4 +1,4 @@
-/* Copyright (c) 2013-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2013-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -36,6 +36,9 @@ struct importer_new_mail {
 	uint32_t local_uid;
 	/* the original remote UID, or 0 if exists only remotely */
 	uint32_t remote_uid;
+	/* UID for the mail in the virtual \All mailbox */
+	uint32_t virtual_all_uid;
+
 	unsigned int uid_in_local:1;
 	unsigned int uid_is_usable:1;
 	unsigned int skip:1;
@@ -58,11 +61,20 @@ struct dsync_mailbox_importer {
 	uint32_t remote_uid_next;
 	uint32_t remote_first_recent_uid;
 	uint64_t remote_highest_modseq, remote_highest_pvt_modseq;
+	time_t sync_since_timestamp;
+
+	enum mail_flags sync_flag;
+	const char *sync_keyword;
+	bool sync_flag_dontwant;
 
 	struct mailbox_transaction_context *trans, *ext_trans;
 	struct mail_search_context *search_ctx;
 	struct mail *mail, *ext_mail;
 
+	struct mailbox *virtual_all_box;
+	struct mailbox_transaction_context *virtual_trans;
+	struct mail *virtual_mail;
+
 	struct mail *cur_mail;
 	const char *cur_guid;
 	const char *cur_hdr_hash;
@@ -107,9 +119,10 @@ struct dsync_mailbox_importer {
 	unsigned int delete_mailbox:1;
 };
 
-static void dsync_mailbox_save_newmails(struct dsync_mailbox_importer *importer,
+static bool dsync_mailbox_save_newmails(struct dsync_mailbox_importer *importer,
 					const struct dsync_mail *mail,
-					struct importer_new_mail *all_newmails);
+					struct importer_new_mail *all_newmails,
+					bool remote_mail);
 static int dsync_mailbox_import_commit(struct dsync_mailbox_importer *importer,
 				       bool final);
 
@@ -169,6 +182,7 @@ dsync_mailbox_import_transaction_begin(struct dsync_mailbox_importer *importer)
 
 struct dsync_mailbox_importer *
 dsync_mailbox_import_init(struct mailbox *box,
+			  struct mailbox *virtual_all_box,
 			  struct dsync_transaction_log_scan *log_scan,
 			  uint32_t last_common_uid,
 			  uint64_t last_common_modseq,
@@ -177,6 +191,7 @@ dsync_mailbox_import_init(struct mailbox *box,
 			  uint32_t remote_first_recent_uid,
 			  uint64_t remote_highest_modseq,
 			  uint64_t remote_highest_pvt_modseq,
+			  time_t sync_since_timestamp, const char *sync_flag,
 			  enum dsync_mailbox_import_flags flags)
 {
 	struct dsync_mailbox_importer *importer;
@@ -188,6 +203,7 @@ dsync_mailbox_import_init(struct mailbox *box,
 	importer = p_new(pool, struct dsync_mailbox_importer, 1);
 	importer->pool = pool;
 	importer->box = box;
+	importer->virtual_all_box = virtual_all_box;
 	importer->last_common_uid = last_common_uid;
 	importer->last_common_modseq = last_common_modseq;
 	importer->last_common_pvt_modseq = last_common_pvt_modseq;
@@ -197,7 +213,18 @@ dsync_mailbox_import_init(struct mailbox *box,
 	importer->remote_first_recent_uid = remote_first_recent_uid;
 	importer->remote_highest_modseq = remote_highest_modseq;
 	importer->remote_highest_pvt_modseq = remote_highest_pvt_modseq;
+	importer->sync_since_timestamp = sync_since_timestamp;
 	importer->stateful_import = importer->last_common_uid_found;
+	if (sync_flag != NULL) {
+		if (sync_flag[0] == '-') {
+			importer->sync_flag_dontwant = TRUE;
+			sync_flag++;
+		}
+		if (sync_flag[0] == '\\')
+			importer->sync_flag = imap_parse_system_flag(sync_flag);
+		else
+			importer->sync_keyword = p_strdup(pool, sync_flag);
+	}
 
 	hash_table_create(&importer->import_guids, pool, 0, str_hash, strcmp);
 	hash_table_create_direct(&importer->import_uids, pool, 0);
@@ -370,8 +397,10 @@ dsync_attributes_cmp(const struct dsync_mailbox_attribute *attr,
 	return dsync_attributes_cmp_values(attr, local_attr, cmp_r);
 }
 
-int dsync_mailbox_import_attribute(struct dsync_mailbox_importer *importer,
-				   const struct dsync_mailbox_attribute *attr)
+static int
+dsync_mailbox_import_attribute_real(struct dsync_mailbox_importer *importer,
+				    const struct dsync_mailbox_attribute *attr,
+				    const char **result_r)
 {
 	struct dsync_mailbox_attribute *local_attr;
 	struct mail_attribute_value value;
@@ -386,25 +415,31 @@ int dsync_mailbox_import_attribute(struct dsync_mailbox_importer *importer,
 	if (attr->deleted &&
 	    (local_attr == NULL || !DSYNC_ATTR_HAS_VALUE(local_attr))) {
 		/* attribute doesn't exist on either side -> ignore */
+		*result_r = "Nonexistent in both sides";
 		return 0;
 	}
 	if (local_attr == NULL) {
 		/* we haven't seen this locally -> use whatever remote has */
+		*result_r = "Nonexistent locally";
 	} else if (local_attr->modseq <= importer->last_common_modseq &&
 		   attr->modseq > importer->last_common_modseq &&
 		   importer->last_common_modseq > 0) {
 		/* we're doing incremental syncing, and we can see that the
 		   attribute was changed remotely, but not locally -> use it */
+		*result_r = "Changed remotely";
 	} else if (local_attr->modseq > importer->last_common_modseq &&
 		   attr->modseq <= importer->last_common_modseq &&
 		   importer->last_common_modseq > 0) {
 		/* we're doing incremental syncing, and we can see that the
 		   attribute was changed locally, but not remotely -> ignore */
+		*result_r = "Changed locally";
 		ignore = TRUE;
 	} else if (attr->last_change > local_attr->last_change) {
 		/* remote has a newer timestamp -> use it */
+		*result_r = "Remote has newer timestamp";
 	} else if (attr->last_change < local_attr->last_change) {
 		/* remote has an older timestamp -> ignore */
+		*result_r = "Local has newer timestamp";
 		ignore = TRUE;
 	} else {
 		/* the timestamps are the same. now we're down to guessing
@@ -418,17 +453,22 @@ int dsync_mailbox_import_attribute(struct dsync_mailbox_importer *importer,
 		}
 		if (cmp == 0) {
 			/* identical scripts */
+			*result_r = "Unchanged value";
 			return 0;
 		}
 
 		if (attr->modseq > local_attr->modseq) {
 			/* remote has a higher modseq -> use it */
+			*result_r = "Remote has newer modseq";
 		} else if (attr->modseq < local_attr->modseq) {
 			/* remote has an older modseq -> ignore */
+			*result_r = "Local has newer modseq";
 			ignore = TRUE;
+		} else if (cmp < 0) {
+			ignore = TRUE;
+			*result_r = "Value changed, but unknown which is newer - picking local";
 		} else {
-			if (cmp < 0)
-				ignore = TRUE;
+			*result_r = "Value changed, but unknown which is newer - picking remote";
 		}
 	}
 	if (ignore) {
@@ -454,6 +494,18 @@ int dsync_mailbox_import_attribute(struct dsync_mailbox_importer *importer,
 	return 0;
 }
 
+int dsync_mailbox_import_attribute(struct dsync_mailbox_importer *importer,
+				   const struct dsync_mailbox_attribute *attr)
+{
+	const char *result;
+	int ret;
+
+	ret = dsync_mailbox_import_attribute_real(importer, attr, &result);
+	imp_debug(importer, "Import attribute %s: %s", attr->key,
+		  ret < 0 ? "failed" : result);
+	return ret;
+}
+
 static void dsync_mail_error(struct dsync_mailbox_importer *importer,
 			     struct mail *mail, const char *field)
 {
@@ -1188,11 +1240,75 @@ dsync_mailbox_import_flag_change(struct dsync_mailbox_importer *importer,
 		mail_update_pvt_modseq(mail, new_modseq);
 }
 
+static bool
+dsync_mail_change_have_keyword(const struct dsync_mail_change *change,
+			       const char *keyword)
+{
+	const char *const *strp;
+
+	if (!array_is_created(&change->keyword_changes))
+		return FALSE;
+
+	array_foreach(&change->keyword_changes, strp) {
+		switch ((*strp)[0]) {
+		case KEYWORD_CHANGE_FINAL:
+		case KEYWORD_CHANGE_ADD_AND_FINAL:
+			if (strcasecmp((*strp)+1, keyword) == 0)
+				return TRUE;
+			break;
+		default:
+			break;
+		}
+	}
+	return FALSE;
+}
+
+static bool
+dsync_mailbox_import_want_change(struct dsync_mailbox_importer *importer,
+				 const struct dsync_mail_change *change,
+				 const char **result_r)
+{
+	if (importer->sync_since_timestamp > 0) {
+		i_assert(change->received_timestamp > 0);
+		if (change->received_timestamp < importer->sync_since_timestamp) {
+			/* mail has too old timestamp - skip it */
+			*result_r = "Ignoring missing local mail with too old timestamp";
+			return FALSE;
+		}
+	}
+	if (importer->sync_flag != 0) {
+		bool have_flag = (change->final_flags & importer->sync_flag) != 0;
+
+		if (have_flag && importer->sync_flag_dontwant) {
+			*result_r = "Ignoring missing local mail that doesn't have wanted flags";
+			return FALSE;
+		}
+		if (!have_flag && !importer->sync_flag_dontwant) {
+			*result_r = "Ignoring missing local mail that has unwanted flags";
+			return FALSE;
+		}
+	}
+	if (importer->sync_keyword != NULL) {
+		bool have_kw = dsync_mail_change_have_keyword(change, importer->sync_keyword);
+
+		if (have_kw && importer->sync_flag_dontwant) {
+			*result_r = "Ignoring missing local mail that doesn't have wanted keywords";
+			return FALSE;
+		}
+		if (!have_kw && !importer->sync_flag_dontwant) {
+			*result_r = "Ignoring missing local mail that has unwanted keywords";
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 static void
 dsync_mailbox_import_save(struct dsync_mailbox_importer *importer,
 			  const struct dsync_mail_change *change)
 {
 	struct dsync_mail_change *save;
+	const char *result;
 
 	i_assert(change->guid != NULL);
 
@@ -1203,6 +1319,8 @@ dsync_mailbox_import_save(struct dsync_mailbox_importer *importer,
 		dsync_mailbox_import_flag_change(importer, change);
 		return;
 	}
+	if (!dsync_mailbox_import_want_change(importer, change, &result))
+		return;
 
 	save = p_new(importer->pool, struct dsync_mail_change, 1);
 	dsync_mail_change_dup(importer->pool, change, save);
@@ -1435,6 +1553,10 @@ dsync_mailbox_find_common_uid(struct dsync_mailbox_importer *importer,
 {
 	int ret;
 
+	i_assert(importer->sync_since_timestamp == 0 ||
+		 change->received_timestamp > 0 ||
+		 change->type == DSYNC_MAIL_CHANGE_TYPE_EXPUNGE);
+
 	/* try to find the matching local mail */
 	if (!importer_next_mail(importer, change->uid)) {
 		/* no more local mails. we can still try to match
@@ -1446,7 +1568,9 @@ dsync_mailbox_find_common_uid(struct dsync_mailbox_importer *importer,
 			return;
 		}
 		i_assert(change->guid != NULL);
-		if (importer->local_uid_next <= change->uid) {
+		if (!dsync_mailbox_import_want_change(importer, change, result_r))
+			;
+		else if (importer->local_uid_next <= change->uid) {
 			dsync_mailbox_common_uid_found(importer);
 			*result_r = "Mail's UID is above local UIDNEXT";
 		} else if (importer->revert_local_changes) {
@@ -1485,6 +1609,9 @@ dsync_mailbox_find_common_uid(struct dsync_mailbox_importer *importer,
 		}
 		return;
 	}
+	/* mail exists remotely, but doesn't exist locally. */
+	if (!dsync_mailbox_import_want_change(importer, change, result_r))
+		return;
 	if (importer->revert_local_changes &&
 	    change->type != DSYNC_MAIL_CHANGE_TYPE_EXPUNGE) {
 		dsync_mailbox_revert_missing(importer, change);
@@ -1602,6 +1729,8 @@ dsync_mailbox_import_assign_new_uids(struct dsync_mailbox_importer *importer)
 			new_uid = newmail->link->final_uid;
 		} else {
 			new_uid = common_uid_next++;
+			imp_debug(importer, "UID %u isn't usable, assigning new UID %u",
+				  newmail->final_uid, new_uid);
 		}
 
 		newmail->final_uid = new_uid;
@@ -1616,17 +1745,17 @@ dsync_mailbox_import_assign_new_uids(struct dsync_mailbox_importer *importer)
 
 static int
 dsync_mailbox_import_local_uid(struct dsync_mailbox_importer *importer,
-			       uint32_t uid, const char *guid,
+			       struct mail *mail, uint32_t uid, const char *guid,
 			       struct dsync_mail *dmail_r)
 {
 	const char *error_field, *errstr;
 	enum mail_error error;
 
-	if (!mail_set_uid(importer->mail, uid))
+	if (!mail_set_uid(mail, uid))
 		return 0;
 
-	if (dsync_mail_fill(importer->mail, TRUE, dmail_r, &error_field) < 0) {
-		errstr = mailbox_get_last_error(importer->mail->box, &error);
+	if (dsync_mail_fill(mail, TRUE, dmail_r, &error_field) < 0) {
+		errstr = mailbox_get_last_error(mail->box, &error);
 		if (error == MAIL_ERROR_EXPUNGED)
 			return 0;
 
@@ -1784,17 +1913,36 @@ dsync_mailbox_import_try_local(struct dsync_mailbox_importer *importer,
 	   other instances */
 	local_n = 0; seq_range_array_iter_init(&local_iter, local_uids);
 	while (seq_range_array_iter_nth(&local_iter, local_n++, &local_uid)) {
-		if (dsync_mailbox_import_local_uid(importer, local_uid,
-						   all_newmails->guid,
+		if (dsync_mailbox_import_local_uid(importer, importer->mail,
+						   local_uid, all_newmails->guid,
 						   &dmail) > 0) {
-			dsync_mailbox_save_newmails(importer, &dmail,
-						    all_newmails);
-			return TRUE;
+			if (dsync_mailbox_save_newmails(importer, &dmail,
+							all_newmails, FALSE))
+				return TRUE;
 		}
 	}
 	return FALSE;
 }
 
+static bool
+dsync_mailbox_import_try_virtual_all(struct dsync_mailbox_importer *importer,
+				     struct importer_new_mail *all_newmails)
+{
+	struct dsync_mail dmail;
+
+	if (all_newmails->virtual_all_uid == 0)
+		return FALSE;
+
+	if (dsync_mailbox_import_local_uid(importer, importer->virtual_mail,
+					   all_newmails->virtual_all_uid,
+					   all_newmails->guid, &dmail) > 0) {
+		if (dsync_mailbox_save_newmails(importer, &dmail,
+						all_newmails, FALSE))
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static bool
 dsync_mailbox_import_handle_mail(struct dsync_mailbox_importer *importer,
 				 struct importer_new_mail *all_newmails)
@@ -1827,7 +1975,8 @@ dsync_mailbox_import_handle_mail(struct dsync_mailbox_importer *importer,
 	i_assert(array_count(&wanted_uids) > 0);
 
 	if (!dsync_mailbox_import_try_local(importer, all_newmails,
-					    &local_uids, &wanted_uids)) {
+					    &local_uids, &wanted_uids) &&
+	    !dsync_mailbox_import_try_virtual_all(importer, all_newmails)) {
 		/* no local instance. request from remote */
 		IMPORTER_DEBUG_CHANGE(importer);
 		if (importer->want_mail_requests) {
@@ -1842,6 +1991,50 @@ dsync_mailbox_import_handle_mail(struct dsync_mailbox_importer *importer,
 	return TRUE;
 }
 
+static void
+dsync_mailbox_import_find_virtual_uids(struct dsync_mailbox_importer *importer)
+{
+	struct mail_search_context *search_ctx;
+	struct mail_search_args *search_args;
+	struct importer_new_mail *newmail;
+	struct mail *mail;
+	const char *guid;
+
+	if (mailbox_sync(importer->virtual_all_box, 0) < 0) {
+		i_error("Couldn't sync \\All mailbox '%s': %s",
+			mailbox_get_vname(importer->virtual_all_box),
+			mailbox_get_last_error(importer->virtual_all_box, NULL));
+		return;
+	}
+
+	search_args = mail_search_build_init();
+	mail_search_build_add_all(search_args);
+
+	importer->virtual_trans =
+		mailbox_transaction_begin(importer->virtual_all_box,
+					  MAILBOX_TRANSACTION_FLAG_SYNC);
+	search_ctx = mailbox_search_init(importer->virtual_trans, search_args,
+					 NULL, MAIL_FETCH_GUID, NULL);
+	mail_search_args_unref(&search_args);
+
+	while (mailbox_search_next(search_ctx, &mail)) {
+		if (mail_get_special(mail, MAIL_FETCH_GUID, &guid) < 0) {
+			/* ignore errors */
+			continue;
+		}
+		newmail = hash_table_lookup(importer->import_guids, guid);
+		if (newmail != NULL && newmail->virtual_all_uid == 0)
+			newmail->virtual_all_uid = mail->uid;
+	}
+	if (mailbox_search_deinit(&search_ctx) < 0) {
+		i_error("Couldn't search \\All mailbox '%s': %s",
+			mailbox_get_vname(importer->virtual_all_box),
+			mailbox_get_last_error(importer->virtual_all_box, NULL));
+	}
+
+	importer->virtual_mail = mail_alloc(importer->virtual_trans, 0, NULL);
+}
+
 static void
 dsync_mailbox_import_handle_local_mails(struct dsync_mailbox_importer *importer)
 {
@@ -1850,6 +2043,12 @@ dsync_mailbox_import_handle_local_mails(struct dsync_mailbox_importer *importer)
 	void *key2;
 	struct importer_new_mail *mail;
 
+	if (importer->virtual_all_box != NULL &&
+	    hash_table_count(importer->import_guids) > 0) {
+		/* find UIDs in \All mailbox for all wanted GUIDs. */
+		dsync_mailbox_import_find_virtual_uids(importer);
+	}
+
 	iter = hash_table_iterate_init(importer->import_guids);
 	while (hash_table_iterate(iter, importer->import_guids, &key, &mail)) {
 		T_BEGIN {
@@ -2017,11 +2216,12 @@ dsync_mailbox_save_init(struct dsync_mailbox_importer *importer,
 	return save_ctx;
 }
 
-static void
+static bool
 dsync_mailbox_save_body(struct dsync_mailbox_importer *importer,
 			const struct dsync_mail *mail,
 			struct importer_new_mail *newmail,
-			struct importer_new_mail **all_newmails_forcopy)
+			struct importer_new_mail **all_newmails_forcopy,
+			bool remote_mail)
 {
 	struct mail_save_context *save_ctx;
 	struct istream *input;
@@ -2048,9 +2248,14 @@ dsync_mailbox_save_body(struct dsync_mailbox_importer *importer,
 	if (ret > 0) {
 		i_assert(save_ctx == NULL);
 		dsync_mailbox_import_saved_uid(importer, newmail->final_uid);
-		return;
+		return TRUE;
 	}
 	/* fallback to saving from remote stream */
+	if (!remote_mail) {
+		/* the mail isn't remote yet. we were just trying to copy a
+		   local mail to avoid downloading the remote mail. */
+		return FALSE;
+	}
 	if (mail->minimal_fields) {
 		struct dsync_mail mail2;
 		const char *error_field;
@@ -2065,7 +2270,7 @@ dsync_mailbox_save_body(struct dsync_mailbox_importer *importer,
 				mailbox_get_last_error(importer->box, NULL));
 			importer->failed = TRUE;
 			mailbox_save_cancel(&save_ctx);
-			return;
+			return TRUE;
 		}
 		dsync_mailbox_save_set_nonminimal(save_ctx, &mail2);
 		input = mail2.input;
@@ -2076,7 +2281,7 @@ dsync_mailbox_save_body(struct dsync_mailbox_importer *importer,
 	if (input == NULL) {
 		/* it was just expunged in remote, skip it */
 		mailbox_save_cancel(&save_ctx);
-		return;
+		return TRUE;
 	}
 
 	i_stream_seek(input, 0);
@@ -2085,7 +2290,7 @@ dsync_mailbox_save_body(struct dsync_mailbox_importer *importer,
 			mailbox_get_vname(importer->box),
 			mailbox_get_last_error(importer->box, NULL));
 		importer->failed = TRUE;
-		return;
+		return TRUE;
 	}
 	while ((ret = i_stream_read(input)) > 0 || ret == -2) {
 		if (mailbox_save_continue(save_ctx) < 0) {
@@ -2120,25 +2325,31 @@ dsync_mailbox_save_body(struct dsync_mailbox_importer *importer,
 						       newmail->final_uid);
 		}
 	}
+	return TRUE;
 }
 
-static void dsync_mailbox_save_newmails(struct dsync_mailbox_importer *importer,
+static bool dsync_mailbox_save_newmails(struct dsync_mailbox_importer *importer,
 					const struct dsync_mail *mail,
-					struct importer_new_mail *all_newmails)
+					struct importer_new_mail *all_newmails,
+					bool remote_mail)
 {
 	struct importer_new_mail *newmail, *all_newmails_forcopy;
+	bool ret = TRUE;
 
 	/* if all_newmails list is large, avoid scanning through the
 	   uninteresting ones for each newmail */
 	all_newmails_forcopy = all_newmails;
 
 	/* save all instances of the message */
-	for (newmail = all_newmails; newmail != NULL; newmail = newmail->next) {
+	for (newmail = all_newmails; newmail != NULL && ret; newmail = newmail->next) {
 		if (!newmail->skip) T_BEGIN {
-			dsync_mailbox_save_body(importer, mail, newmail,
-						&all_newmails_forcopy);
+			if (!dsync_mailbox_save_body(importer, mail, newmail,
+						     &all_newmails_forcopy,
+						     remote_mail))
+				ret = FALSE;
 		} T_END;
 	}
+	return ret;
 }
 
 void dsync_mailbox_import_mail(struct dsync_mailbox_importer *importer,
@@ -2177,7 +2388,8 @@ void dsync_mailbox_import_mail(struct dsync_mailbox_importer *importer,
 				  POINTER_CAST(mail->uid));
 	}
 	importer->import_pos++;
-	dsync_mailbox_save_newmails(importer, mail, all_newmails);
+	if (!dsync_mailbox_save_newmails(importer, mail, all_newmails, TRUE))
+		i_unreached();
 }
 
 static int
@@ -2487,6 +2699,11 @@ int dsync_mailbox_import_deinit(struct dsync_mailbox_importer **_importer,
 	if (dsync_mailbox_import_finish(importer, changes_during_sync_r) < 0)
 		importer->failed = TRUE;
 
+	if (importer->virtual_mail != NULL)
+		mail_free(&importer->virtual_mail);
+	if (importer->virtual_trans != NULL)
+		(void)mailbox_transaction_commit(&importer->virtual_trans);
+
 	hash_table_destroy(&importer->import_guids);
 	hash_table_destroy(&importer->import_uids);
 	array_free(&importer->maybe_expunge_uids);
