@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -155,6 +155,16 @@ void fs_deinit(struct fs **_fs)
 	str_free(&last_error);
 }
 
+struct fs *fs_get_parent(struct fs *fs)
+{
+	return fs->parent;
+}
+
+const char *fs_get_driver(struct fs *fs)
+{
+	return fs->name;
+}
+
 const char *fs_get_root_driver(struct fs *fs)
 {
 	while (fs->parent != NULL)
@@ -259,12 +269,35 @@ int fs_get_metadata(struct fs_file *file,
 		fs_set_error(file->fs, "Metadata not supported by backend");
 		return -1;
 	}
+	if (!file->read_or_prefetch_counted &&
+	    !file->lookup_metadata_counted) {
+		file->lookup_metadata_counted = TRUE;
+		file->fs->stats.lookup_metadata_count++;
+	}
 	T_BEGIN {
 		ret = file->fs->v.get_metadata(file, metadata_r);
 	} T_END;
 	return ret;
 }
 
+int fs_lookup_metadata(struct fs_file *file, const char *key,
+		       const char **value_r)
+{
+	const ARRAY_TYPE(fs_metadata) *metadata;
+	const struct fs_metadata *md;
+
+	if (fs_get_metadata(file, &metadata) < 0)
+		return -1;
+	array_foreach(metadata, md) {
+		if (strcmp(md->key, key) == 0) {
+			*value_r = md->value;
+			return 1;
+		}
+	}
+	*value_r = NULL;
+	return 0;
+}
+
 const char *fs_file_path(struct fs_file *file)
 {
 	return file->fs->v.get_path == NULL ? file->path :
@@ -308,6 +341,10 @@ bool fs_prefetch(struct fs_file *file, uoff_t length)
 {
 	bool ret;
 
+	if (!file->read_or_prefetch_counted) {
+		file->read_or_prefetch_counted = TRUE;
+		file->fs->stats.prefetch_count++;
+	}
 	T_BEGIN {
 		ret = file->fs->v.prefetch(file, length);
 	} T_END;
@@ -346,6 +383,11 @@ ssize_t fs_read(struct fs_file *file, void *buf, size_t size)
 {
 	int ret;
 
+	if (!file->read_or_prefetch_counted) {
+		file->read_or_prefetch_counted = TRUE;
+		file->fs->stats.read_count++;
+	}
+
 	if (file->fs->v.read != NULL) {
 		T_BEGIN {
 			ret = file->fs->v.read(file, buf, size);
@@ -366,6 +408,11 @@ struct istream *fs_read_stream(struct fs_file *file, size_t max_buffer_size)
 	ssize_t ret;
 	bool want_seekable = FALSE;
 
+	if (!file->read_or_prefetch_counted) {
+		file->read_or_prefetch_counted = TRUE;
+		file->fs->stats.read_count++;
+	}
+
 	if (file->seekable_input != NULL) {
 		i_stream_seek(file->seekable_input, 0);
 		i_stream_ref(file->seekable_input);
@@ -448,6 +495,7 @@ int fs_write(struct fs_file *file, const void *data, size_t size)
 {
 	int ret;
 
+	file->fs->stats.write_count++;
 	if (file->fs->v.write != NULL) {
 		T_BEGIN {
 			ret = file->fs->v.write(file, data, size);
@@ -462,6 +510,7 @@ int fs_write(struct fs_file *file, const void *data, size_t size)
 
 struct ostream *fs_write_stream(struct fs_file *file)
 {
+	file->fs->stats.write_count++;
 	T_BEGIN {
 		file->fs->v.write_stream(file);
 	} T_END;
@@ -579,6 +628,7 @@ int fs_exists(struct fs_file *file)
 		else
 			return errno == ENOENT ? 0 : -1;
 	}
+	file->fs->stats.exists_count++;
 	T_BEGIN {
 		ret = file->fs->v.exists(file);
 	} T_END;
@@ -589,6 +639,11 @@ int fs_stat(struct fs_file *file, struct stat *st_r)
 {
 	int ret;
 
+	if (!file->read_or_prefetch_counted &&
+	    !file->lookup_metadata_counted && !file->stat_counted) {
+		file->stat_counted = TRUE;
+		file->fs->stats.stat_count++;
+	}
 	T_BEGIN {
 		ret = file->fs->v.stat(file, st_r);
 	} T_END;
@@ -597,6 +652,10 @@ int fs_stat(struct fs_file *file, struct stat *st_r)
 
 int fs_default_copy(struct fs_file *src, struct fs_file *dest)
 {
+	/* we're going to be counting this as read+write, so remove the
+	   copy_count we just added */
+	dest->fs->stats.copy_count--;
+
 	if (dest->copy_src != NULL) {
 		i_assert(src == NULL || src == dest->copy_src);
 		if (dest->copy_output == NULL) {
@@ -647,6 +706,7 @@ int fs_copy(struct fs_file *src, struct fs_file *dest)
 
 	i_assert(src->fs == dest->fs);
 
+	dest->fs->stats.copy_count++;
 	T_BEGIN {
 		ret = src->fs->v.copy(src, dest);
 	} T_END;
@@ -673,6 +733,7 @@ int fs_rename(struct fs_file *src, struct fs_file *dest)
 
 	i_assert(src->fs == dest->fs);
 
+	dest->fs->stats.rename_count++;
 	T_BEGIN {
 		ret = src->fs->v.rename(src, dest);
 	} T_END;
@@ -683,6 +744,7 @@ int fs_delete(struct fs_file *file)
 {
 	int ret;
 
+	file->fs->stats.delete_count++;
 	T_BEGIN {
 		ret = file->fs->v.delete_file(file);
 	} T_END;
@@ -694,6 +756,10 @@ fs_iter_init(struct fs *fs, const char *path, enum fs_iter_flags flags)
 {
 	struct fs_iter *iter;
 
+	i_assert((flags & FS_ITER_FLAG_OBJECTIDS) == 0 ||
+		 (fs_get_properties(fs) & FS_PROPERTY_OBJECTIDS) != 0);
+
+	fs->stats.iter_count++;
 	T_BEGIN {
 		iter = fs->v.iter_init(fs, path, flags);
 	} T_END;
@@ -737,6 +803,11 @@ bool fs_iter_have_more(struct fs_iter *iter)
 	return iter->async_have_more;
 }
 
+const struct fs_stats *fs_get_stats(struct fs *fs)
+{
+	return &fs->stats;
+}
+
 void fs_set_error(struct fs *fs, const char *fmt, ...)
 {
 	va_list args;
