@@ -1,4 +1,4 @@
-/* Copyright (c) 2013-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2013-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "net.h"
@@ -176,10 +176,9 @@ bool http_client_connection_is_ready(struct http_client_connection *conn)
 		/* Active ioloop is different from what we saw earlier;
 		   we may have missed a disconnection event on this connection.
 		   Verify status by reading from connection. */
-		if ((ret=i_stream_read(conn->conn.input)) < 0) {
+		if ((ret=i_stream_read(conn->conn.input)) == -1) {
 			int stream_errno = conn->conn.input->stream_errno;
 
-			i_assert(ret != -2);
 			i_assert(conn->conn.input->stream_errno != 0 || conn->conn.input->eof);
 			http_client_connection_abort_temp_error(&conn,
 				HTTP_CLIENT_REQUEST_ERROR_CONNECTION_LOST,
@@ -209,9 +208,6 @@ void http_client_connection_check_idle(struct http_client_connection *conn)
 {
 	unsigned int timeout, count;
 
-	i_assert(!conn->close_indicated);
-	i_assert(!conn->output_broken);
-
 	if (conn->connected &&
 		array_is_created(&conn->request_wait_list) &&
 		array_count(&conn->request_wait_list) == 0 &&
@@ -445,6 +441,7 @@ static void http_client_payload_destroyed(struct http_client_request *req)
 	req->conn = NULL;
 	conn->incoming_payload = NULL;
 	conn->pending_request = NULL;
+	http_client_connection_ref(conn);
 	http_client_request_finish(&req);
 
 	/* room for new requests */
@@ -461,6 +458,7 @@ static void http_client_payload_destroyed(struct http_client_request *req)
 
 	i_assert(req != NULL);
 	http_client_request_unref(&req);
+	http_client_connection_unref(&conn);
 }
 
 static bool
@@ -1144,42 +1142,6 @@ http_client_connect_tunnel_timeout(struct http_client_connection *conn)
 	http_client_connection_close(&conn);
 }
 
-// FIXME: put something like this in lib/connection.c
-static void
-_connection_init_from_streams(struct connection_list *list,
-			    struct connection *conn, const char *name,
-			    struct istream *input, struct ostream *output)
-{
-	i_assert(name != NULL);
-
-	conn->list = list;
-	conn->name = i_strdup(name);
-	conn->fd_in = i_stream_get_fd(input);
-	conn->fd_out = o_stream_get_fd(output);
-
-	i_assert(conn->fd_in >= 0);
-	i_assert(conn->fd_out >= 0);
-	i_assert(conn->io == NULL);
-	i_assert(conn->input == NULL);
-	i_assert(conn->output == NULL);
-	i_assert(conn->to == NULL);
-
-	conn->input = input;
-	i_stream_set_name(conn->input, conn->name);
-
-	conn->output = output;
-	o_stream_set_no_error_handling(conn->output, TRUE);
-	o_stream_set_name(conn->output, conn->name);
-
-	conn->io = io_add_istream(conn->input, *list->v.input, conn);
-	
-	DLLIST_PREPEND(&list->connections, conn);
-	list->connections_count++;
-
-	if (list->v.client_connected != NULL)
-		list->v.client_connected(conn, TRUE);
-}
-
 static void
 http_client_connection_tunnel_response(const struct http_response *response,
 			       struct http_client_connection *conn)
@@ -1198,7 +1160,7 @@ http_client_connection_tunnel_response(const struct http_response *response,
 	http_client_request_start_tunnel(conn->connect_request, &tunnel);
 	conn->connect_request = NULL;
 
-	_connection_init_from_streams
+	connection_init_from_streams
 		(conn->client->conn_list, &conn->conn, name, tunnel.input, tunnel.output);
 	conn->connect_initialized = TRUE;
 }
