@@ -14,6 +14,7 @@
 #define HTTP_CLIENT_CONTINUE_TIMEOUT_MSECS (1000*2)
 #define HTTP_CLIENT_DEFAULT_REQUEST_TIMEOUT_MSECS (1000*60*5)
 #define HTTP_CLIENT_DEFAULT_BACKOFF_TIME_MSECS (100)
+#define HTTP_CLIENT_DEFAULT_BACKOFF_MAX_TIME_MSECS (1000*60)
 
 enum http_response_payload_type;
 
@@ -52,6 +53,8 @@ struct http_client_request {
 	unsigned int refcount;
 	const char *label;
 
+	struct http_client_request *prev, *next;
+
 	const char *method, *target;
 	struct http_url origin_url;
 
@@ -139,10 +142,12 @@ struct http_client_connection {
 	unsigned int connected:1;           /* connection is connected */
 	unsigned int tunneling:1;          /* last sent request turns this
 	                                      connection into tunnel */
+	unsigned int connect_initialized:1; /* connection was initialized */
 	unsigned int connect_succeeded:1;
 	unsigned int closing:1;
 	unsigned int close_indicated:1;
 	unsigned int output_locked:1;       /* output is locked; no pipelining */
+	unsigned int output_broken:1;       /* output is broken; no more requests */
 	unsigned int payload_continue:1;    /* received 100-continue for current
 	                                        request */
 	unsigned int in_req_callback:1;  /* performin request callback (busy) */
@@ -245,7 +250,8 @@ struct http_client {
 	struct http_client_host *hosts_list;
 	HASH_TABLE_TYPE(http_client_peer) peers;
 	struct http_client_peer *peers_list;
-	unsigned int pending_requests;
+	struct http_client_request *requests_list;
+	unsigned int requests_count;
 };
 
 int http_client_init_ssl_ctx(struct http_client *client, const char **error_r);
@@ -268,8 +274,6 @@ void http_client_request_connect_callback(struct http_client_request *req,
 void http_client_request_resubmit(struct http_client_request *req);
 void http_client_request_retry(struct http_client_request *req,
 	unsigned int status, const char *error);
-void http_client_request_send_error(struct http_client_request *req,
-			       unsigned int status, const char *error);
 void http_client_request_error_delayed(struct http_client_request **_req);
 void http_client_request_error(struct http_client_request *req,
 	unsigned int status, const char *error);
@@ -283,6 +287,7 @@ struct http_client_connection *
 	http_client_connection_create(struct http_client_peer *peer);
 void http_client_connection_ref(struct http_client_connection *conn);
 void http_client_connection_unref(struct http_client_connection **_conn);
+void http_client_connection_close(struct http_client_connection **_conn);
 int http_client_connection_output(struct http_client_connection *conn);
 unsigned int
 http_client_connection_count_pending(struct http_client_connection *conn);
@@ -346,9 +351,8 @@ http_client_queue_requests_pending(struct http_client_queue *queue,
 void
 http_client_queue_connection_success(struct http_client_queue *queue,
 					 const struct http_client_peer_addr *addr);
-bool
-http_client_queue_connection_failure(struct http_client_queue *queue,
-	const struct http_client_peer_addr *addr, const char *reason);
+void http_client_queue_connection_failure(struct http_client_queue *queue,
+ 	const struct http_client_peer_addr *addr, const char *reason);
 void http_client_queue_switch_ioloop(struct http_client_queue *queue);
 
 struct http_client_host *
