@@ -23,6 +23,7 @@ enum imapc_capability {
 	IMAPC_CAPABILITY_CONDSTORE	= 0x100,
 	IMAPC_CAPABILITY_NAMESPACE	= 0x200,
 	IMAPC_CAPABILITY_UNSELECT	= 0x400,
+	IMAPC_CAPABILITY_ESEARCH	= 0x800,
 
 	IMAPC_CAPABILITY_IMAP4REV1	= 0x40000000
 };
@@ -53,6 +54,12 @@ enum imapc_client_ssl_mode {
 #define IMAPC_DEFAULT_CONNECT_TIMEOUT_MSECS (1000*30)
 #define IMAPC_DEFAULT_COMMAND_TIMEOUT_MSECS (1000*60*5)
 
+struct imapc_throttling_settings {
+	unsigned int init_msecs;
+	unsigned int max_msecs;
+	unsigned int shrink_min_msecs;
+};
+
 struct imapc_client_settings {
 	const char *host;
 	unsigned int port;
@@ -60,6 +67,9 @@ struct imapc_client_settings {
 	const char *master_user;
 	const char *username;
 	const char *password;
+	/* Space-separated list of SASL mechanisms to try (in the specified
+	   order). The default is to use only LOGIN command or SASL PLAIN. */
+	const char *sasl_mechanisms;
 	unsigned int max_idle_time;
 
 	const char *dns_client_socket_path;
@@ -78,6 +88,8 @@ struct imapc_client_settings {
 	/* Timeout for IMAP commands. Reset every time more data is being
 	   sent or received. 0 = default. */
 	unsigned int cmd_timeout_msecs;
+
+	struct imapc_throttling_settings throttle_set;
 };
 
 struct imapc_command_reply {
@@ -146,11 +158,14 @@ imapc_client_cmd(struct imapc_client *client,
 		 imapc_command_callback_t *callback, void *context);
 void imapc_command_set_flags(struct imapc_command *cmd,
 			     enum imapc_command_flags flags);
+bool imapc_command_connection_is_selected(struct imapc_command *cmd);
 void imapc_command_send(struct imapc_command *cmd, const char *cmd_str);
 void imapc_command_sendf(struct imapc_command *cmd, const char *cmd_fmt, ...)
 	ATTR_FORMAT(2, 3);
 void imapc_command_sendvf(struct imapc_command *cmd,
 			  const char *cmd_fmt, va_list args) ATTR_FORMAT(2, 0);
+const char *imapc_command_get_tag(struct imapc_command *cmd);
+void imapc_command_abort(struct imapc_command **cmd);
 
 void imapc_client_register_untagged(struct imapc_client *client,
 				    imapc_untagged_callback_t *callback,
