@@ -1,4 +1,4 @@
-/* Copyright (c) 2011-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2011-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "str.h"
@@ -146,7 +146,19 @@ imapc_mail_delayed_send_or_merge(struct imapc_mail *mail, string_t *str)
 	}
 	array_append(&mbox->pending_fetch_request->mails, &mail, 1);
 
-	if (mbox->to_pending_fetch_send == NULL) {
+	if (mbox->to_pending_fetch_send == NULL &&
+	    array_count(&mbox->pending_fetch_request->mails) >
+	    			mbox->box.storage->set->mail_prefetch_count) {
+		/* we're now prefetching the maximum number of mails. this
+		   most likely means that we need to flush out the command now
+		   before sending anything else. delay it a little bit though
+		   in case the sending code doesn't actually use
+		   mail_prefetch_count and wants to fetch more.
+
+		   note that we don't want to add this timeout too early,
+		   because we want to optimize the maximum number of messages
+		   placed into a single FETCH. even without timeout the command
+		   gets flushed by imapc_mail_fetch() call. */
 		mbox->to_pending_fetch_send =
 			timeout_add_short(0, imapc_mail_fetch_flush, mbox);
 	}
@@ -171,7 +183,7 @@ imapc_mail_send_fetch(struct mail *_mail, enum mail_fetch_field fields,
 	if (headers_have_subset(mail->fetching_headers, headers))
 		headers = NULL;
 	if (fields == 0 && headers == NULL)
-		return 0;
+		return mail->fetch_sent ? 0 : 1;
 
 	if (!_mail->saving) {
 		/* if we already know that the mail is expunged,
@@ -229,6 +241,7 @@ imapc_mail_send_fetch(struct mail *_mail, enum mail_fetch_field fields,
 	pool_ref(mail->imail.mail.pool);
 	mail->fetching_fields |= fields;
 	mail->fetch_count++;
+	mail->fetch_sent = FALSE;
 
 	imapc_mail_delayed_send_or_merge(mail, str);
 	return 1;
@@ -302,7 +315,7 @@ bool imapc_mail_prefetch(struct mail *_mail)
 	imapc_mail_update_access_parts(&mail->imail);
 
 	fields = imapc_mail_get_wanted_fetch_fields(mail);
-	if (fields != 0) T_BEGIN {
+	if (fields != 0 || data->wanted_headers != NULL) T_BEGIN {
 		if (imapc_mail_send_fetch(_mail, fields,
 					  data->wanted_headers == NULL ? NULL :
 					  data->wanted_headers->name) > 0)
@@ -365,7 +378,8 @@ int imapc_mail_fetch(struct mail *_mail, enum mail_fetch_field fields,
 	/* we'll continue waiting until we've got all the fields we wanted,
 	   or until all FETCH replies have been received (i.e. some FETCHes
 	   failed) */
-	imapc_mail_fetch_flush(mbox);
+	if (ret > 0)
+		imapc_mail_fetch_flush(mbox);
 	while (imail->fetch_count > 0 &&
 	       (!imapc_mail_have_fields(imail, fields) ||
 		!imail->header_list_fetched))
@@ -376,12 +390,16 @@ int imapc_mail_fetch(struct mail *_mail, enum mail_fetch_field fields,
 void imapc_mail_fetch_flush(struct imapc_mailbox *mbox)
 {
 	struct imapc_command *cmd;
+	struct imapc_mail *const *mailp;
 
 	if (mbox->pending_fetch_request == NULL) {
 		i_assert(mbox->to_pending_fetch_send == NULL);
 		return;
 	}
 
+	array_foreach(&mbox->pending_fetch_request->mails, mailp)
+		(*mailp)->fetch_sent = TRUE;
+
 	cmd = imapc_client_mailbox_cmd(mbox->client_box,
 				       imapc_mail_fetch_callback,
 				       mbox->pending_fetch_request);
@@ -391,7 +409,8 @@ void imapc_mail_fetch_flush(struct imapc_mailbox *mbox)
 	imapc_command_send(cmd, str_c(mbox->pending_fetch_cmd));
 
 	mbox->pending_fetch_request = NULL;
-	timeout_remove(&mbox->to_pending_fetch_send);
+	if (mbox->to_pending_fetch_send != NULL)
+		timeout_remove(&mbox->to_pending_fetch_send);
 	str_truncate(mbox->pending_fetch_cmd, 0);
 }
 
