@@ -1,4 +1,4 @@
-/* Copyright (c) 2011-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2011-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -11,6 +11,7 @@
 #include "imapc-msgmap.h"
 #include "imapc-mail.h"
 #include "imapc-list.h"
+#include "imapc-search.h"
 #include "imapc-sync.h"
 #include "imapc-settings.h"
 #include "imapc-storage.h"
@@ -221,6 +222,7 @@ int imapc_storage_client_create(struct mail_namespace *ns,
 		*error_r = "missing imapc_password";
 		return -1;
 	}
+	set.sasl_mechanisms = imapc_set->imapc_sasl_mechanisms;
 	set.max_idle_time = imapc_set->imapc_max_idle_time;
 	set.dns_client_socket_path = *ns->user->set->base_dir == '\0' ? "" :
 		t_strconcat(ns->user->set->base_dir, "/",
@@ -244,6 +246,10 @@ int imapc_storage_client_create(struct mail_namespace *ns,
 		set.ssl_mode = IMAPC_CLIENT_SSL_MODE_NONE;
 	set.ssl_crypto_device = mail_set->ssl_crypto_device;
 
+	set.throttle_set.init_msecs = imapc_set->throttle_init_msecs;
+	set.throttle_set.max_msecs = imapc_set->throttle_max_msecs;
+	set.throttle_set.shrink_min_msecs = imapc_set->throttle_shrink_min_msecs;
+
 	client = i_new(struct imapc_storage_client, 1);
 	client->refcount = 1;
 	i_array_init(&client->untagged_callbacks, 16);
@@ -674,6 +680,8 @@ static void imapc_mailbox_get_selected_status(struct imapc_mailbox *mbox,
 	index_storage_get_open_status(&mbox->box, items, status_r);
 	if ((items & STATUS_PERMANENT_FLAGS) != 0)
 		status_r->permanent_flags = mbox->permanent_flags;
+	if ((items & STATUS_FIRST_RECENT_UID) != 0)
+		status_r->first_recent_uid = mbox->highest_nonrecent_uid + 1;
 }
 
 static int imapc_mailbox_delete(struct mailbox *box)
@@ -733,7 +741,8 @@ static int imapc_mailbox_get_status(struct mailbox *box,
 	if (box->opened) {
 		imapc_mailbox_get_selected_status(mbox, items, status_r);
 	} else if ((items & (STATUS_FIRST_UNSEEN_SEQ | STATUS_KEYWORDS |
-			     STATUS_PERMANENT_FLAGS)) != 0) {
+			     STATUS_PERMANENT_FLAGS |
+			     STATUS_FIRST_RECENT_UID)) != 0) {
 		/* getting these requires opening the mailbox */
 		if (mailbox_open(box) < 0)
 			return -1;
@@ -957,10 +966,10 @@ struct mailbox imapc_mailbox = {
 		index_transaction_rollback,
 		NULL,
 		imapc_mail_alloc,
-		index_storage_search_init,
-		index_storage_search_deinit,
+		imapc_search_init,
+		imapc_search_deinit,
 		index_storage_search_next_nonblock,
-		index_storage_search_next_update_seq,
+		imapc_search_next_update_seq,
 		imapc_save_alloc,
 		imapc_save_begin,
 		imapc_save_continue,
