@@ -5,10 +5,14 @@
 #include "mail-storage-settings.h"
 
 struct module;
+struct stats;
+struct fs_settings;
+struct ssl_iostream_settings;
 struct mail_user;
 
 struct mail_user_vfuncs {
 	void (*deinit)(struct mail_user *user);
+	void (*stats_fill)(struct mail_user *user, struct stats *stats);
 };
 
 struct mail_user {
@@ -16,6 +20,10 @@ struct mail_user {
 	struct mail_user_vfuncs v, *vlast;
 	int refcount;
 
+	/* User's creator if such exists. For example for autocreated shared
+	   mailbox users their creator is the logged in user. */
+	struct mail_user *creator;
+
 	const char *username;
 	/* don't access the home directly. It may be set lazily. */
 	const char *_home;
@@ -155,4 +163,13 @@ bool mail_user_is_path_mounted(struct mail_user *user, const char *path,
 struct mail_storage *
 mail_user_get_storage_class(struct mail_user *user, const char *name);
 
+/* Initialize fs_settings from mail_user settings. */
+void mail_user_init_fs_settings(struct mail_user *user,
+				struct fs_settings *fs_set,
+				struct ssl_iostream_settings *ssl_set);
+
+/* Fill statistics for user. By default there are no statistics, so stats
+   plugin must be loaded to have anything filled. */
+void mail_user_stats_fill(struct mail_user *user, struct stats *stats);
+
 #endif
