@@ -1,55 +1,52 @@
 #ifndef MAILBOX_LIST_PRIVATE_H
 #define MAILBOX_LIST_PRIVATE_H
 
+#include "mailbox-log.h"
+#include "mailbox-list-notify.h"
 #include "mail-namespace.h"
 #include "mailbox-list.h"
+#include "mailbox-list-iter.h"
 #include "mail-storage-settings.h"
 
 #define MAILBOX_LIST_NAME_MAILDIRPLUSPLUS "maildir++"
 #define MAILBOX_LIST_NAME_IMAPDIR "imapdir"
 #define MAILBOX_LIST_NAME_FS "fs"
+#define MAILBOX_LIST_NAME_INDEX "index"
+#define MAILBOX_LIST_NAME_NONE "none"
 
 #define MAILBOX_LOG_FILE_NAME "dovecot.mailbox.log"
 
-enum mailbox_log_record_type;
+#define T_MAILBOX_LIST_ERR_NOT_FOUND(list, name) \
+	t_strdup_printf(MAIL_ERRSTR_MAILBOX_NOT_FOUND, \
+			mailbox_list_get_vname(list, name))
+
 struct stat;
 struct dirent;
+struct fs;
 struct imap_match_glob;
 struct mailbox_tree_context;
+struct mailbox_list_notify;
+struct mailbox_list_notify_rec;
 
 #define MAILBOX_INFO_FLAGS_FINISHED(flags) \
 	(((flags) & (MAILBOX_SELECT | MAILBOX_NOSELECT | \
 		     MAILBOX_NONEXISTENT)) != 0)
 
-enum mailbox_dir_create_type {
-	/* Creating a mailbox */
-	MAILBOX_DIR_CREATE_TYPE_MAILBOX,
-	/* Create a \Noselect or a mailbox */
-	MAILBOX_DIR_CREATE_TYPE_TRY_NOSELECT,
-	/* Create a \Noselect or fail */
-	MAILBOX_DIR_CREATE_TYPE_ONLY_NOSELECT
-};
-
 struct mailbox_list_vfuncs {
 	struct mailbox_list *(*alloc)(void);
+	int (*init)(struct mailbox_list *list, const char **error_r);
 	void (*deinit)(struct mailbox_list *list);
 
 	int (*get_storage)(struct mailbox_list **list, const char *vname,
 			   struct mail_storage **storage_r);
-	bool (*is_valid_pattern)(struct mailbox_list *list,
-				 const char *pattern);
-	bool (*is_valid_existing_name)(struct mailbox_list *list,
-				       const char *name);
-	bool (*is_valid_create_name)(struct mailbox_list *list,
-				     const char *name);
 
 	char (*get_hierarchy_sep)(struct mailbox_list *list);
 	const char *(*get_vname)(struct mailbox_list *list,
 				 const char *storage_name);
 	const char *(*get_storage_name)(struct mailbox_list *list,
 					const char *vname);
-	const char *(*get_path)(struct mailbox_list *list, const char *name,
-				enum mailbox_list_path_type type);
+	int (*get_path)(struct mailbox_list *list, const char *name,
+			enum mailbox_list_path_type type, const char **path_r);
 
 	const char *(*get_temp_prefix)(struct mailbox_list *list, bool global);
 	const char *(*join_refpattern)(struct mailbox_list *list,
@@ -77,14 +74,20 @@ struct mailbox_list_vfuncs {
 				     struct mailbox_list *dest_list);
 	int (*set_subscribed)(struct mailbox_list *list,
 			      const char *name, bool set);
-	int (*create_mailbox_dir)(struct mailbox_list *list, const char *name,
-				  enum mailbox_dir_create_type type);
 	int (*delete_mailbox)(struct mailbox_list *list, const char *name);
 	int (*delete_dir)(struct mailbox_list *list, const char *name);
 	int (*delete_symlink)(struct mailbox_list *list, const char *name);
 	int (*rename_mailbox)(struct mailbox_list *oldlist, const char *oldname,
-			      struct mailbox_list *newlist, const char *newname,
-			      bool rename_children);
+			      struct mailbox_list *newlist, const char *newname);
+
+	int (*notify_init)(struct mailbox_list *list,
+			   enum mailbox_list_notify_event mask,
+			   struct mailbox_list_notify **notify_r);
+	int (*notify_next)(struct mailbox_list_notify *notify,
+			   const struct mailbox_list_notify_rec **rec_r);
+	void (*notify_deinit)(struct mailbox_list_notify *notify);
+	void (*notify_wait)(struct mailbox_list_notify *notify,
+			    void (*callback)(void *context), void *context);
 };
 
 struct mailbox_list_module_register {
@@ -110,11 +113,8 @@ struct mailbox_list {
 	const struct mail_storage_settings *mail_set;
 	enum mailbox_list_flags flags;
 
-	/* -1 if not set yet. use mailbox_list_get_permissions() to set them */
-	mode_t file_create_mode, dir_create_mode;
-	gid_t file_create_gid;
-	/* origin (e.g. path) where the file_create_gid was got from */
-	const char *file_create_gid_origin;
+	/* may not be set yet, use mailbox_list_get_permissions() to access */
+	struct mailbox_permissions root_permissions;
 
 	struct mailbox_tree_context *subscriptions;
 	time_t subscriptions_mtime, subscriptions_read_time;
@@ -123,16 +123,18 @@ struct mailbox_list {
 	time_t changelog_timestamp;
 
 	pool_t guid_cache_pool;
-	struct hash_table *guid_cache;
+	HASH_TABLE(uint8_t *, struct mailbox_guid_cache_rec *) guid_cache;
 	bool guid_cache_errors;
 
 	char *error_string;
 	enum mail_error error;
 	bool temporary_error;
 
-	ARRAY_DEFINE(module_contexts, union mailbox_list_module_context *);
+	ARRAY(union mailbox_list_module_context *) module_contexts;
 
 	unsigned int index_root_dir_created:1;
+	unsigned int guid_cache_updated:1;
+	unsigned int guid_cache_invalidated:1;
 };
 
 union mailbox_list_iterate_module_context {
@@ -149,8 +151,7 @@ struct mailbox_list_iterate_context {
 	struct mailbox_list_autocreate_iterate_context *autocreate_ctx;
 	struct mailbox_info specialuse_info;
 
-	ARRAY_DEFINE(module_contexts,
-		     union mailbox_list_iterate_module_context *);
+	ARRAY(union mailbox_list_iterate_module_context *) module_contexts;
 };
 
 struct mailbox_list_iter_update_context {
@@ -180,28 +181,28 @@ const char *mailbox_list_default_get_vname(struct mailbox_list *list,
 					   const char *storage_name);
 const char *mailbox_list_get_unexpanded_path(struct mailbox_list *list,
 					     enum mailbox_list_path_type type);
-const char *
-mailbox_list_get_root_path(const struct mailbox_list_settings *set,
-			   enum mailbox_list_path_type type);
+bool mailbox_list_set_get_root_path(const struct mailbox_list_settings *set,
+				    enum mailbox_list_path_type type,
+				    const char **path_r);
 
 int mailbox_list_delete_index_control(struct mailbox_list *list,
 				      const char *name);
 
 void mailbox_list_iter_update(struct mailbox_list_iter_update_context *ctx,
 			      const char *name);
+int mailbox_list_iter_subscriptions_refresh(struct mailbox_list *list);
 
 bool mailbox_list_name_is_too_large(const char *name, char sep);
 enum mailbox_list_file_type mailbox_list_get_file_type(const struct dirent *d);
+int mailbox_list_dirent_is_alias_symlink(struct mailbox_list *list,
+					 const char *dir_path,
+					 const struct dirent *d);
 bool mailbox_list_try_get_absolute_path(struct mailbox_list *list,
 					const char **name);
-int mailbox_list_create_missing_index_dir(struct mailbox_list *list,
-					  const char *name);
 
 void mailbox_list_add_change(struct mailbox_list *list,
 			     enum mailbox_log_record_type type,
 			     const guid_128_t guid_128);
-int mailbox_list_get_guid_path(struct mailbox_list *list, const char *path,
-			       guid_128_t guid_128_r);
 void mailbox_name_get_sha128(const char *name, guid_128_t guid_128_r);
 
 void mailbox_list_clear_error(struct mailbox_list *list);
@@ -212,4 +213,8 @@ void mailbox_list_set_critical(struct mailbox_list *list, const char *fmt, ...)
 void mailbox_list_set_internal_error(struct mailbox_list *list);
 bool mailbox_list_set_error_from_errno(struct mailbox_list *list);
 
+int mailbox_list_init_fs(struct mailbox_list *list, const char *driver,
+			 const char *args, const char *root_dir,
+			 struct fs **fs_r, const char **error_r);
+
 #endif
