@@ -1,4 +1,4 @@
-/* Copyright (c) 2003-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2003-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -7,41 +7,20 @@
 
 void *array_idx_modifiable_i(struct array *array, unsigned int idx)
 {
-	size_t pos;
-
-	pos = idx * array->element_size;
-	if (pos >= array->buffer->used) {
-		/* index doesn't exist yet, initialize with zero */
-		buffer_append_zero(array->buffer, pos + array->element_size -
-				   array->buffer->used);
-	}
-	return buffer_get_space_unsafe(array->buffer, pos, array->element_size);
+	return buffer_get_space_unsafe(array->buffer, idx * array->element_size,
+				       array->element_size);
 }
 
 void array_idx_set_i(struct array *array, unsigned int idx, const void *data)
 {
-	size_t pos;
-
-	pos = idx * array->element_size;
-	if (pos > array->buffer->used) {
-		/* index doesn't exist yet, initialize with zero */
-		buffer_append_zero(array->buffer, pos - array->buffer->used);
-	}
-	buffer_write(array->buffer, pos, data, array->element_size);
+	buffer_write(array->buffer, idx * array->element_size,
+		     data, array->element_size);
 }
 
 void array_idx_clear_i(struct array *array, unsigned int idx)
 {
-	size_t pos;
-
-	pos = idx * array->element_size;
-	if (pos > array->buffer->used) {
-		/* index doesn't exist yet, initialize with zero */
-		buffer_append_zero(array->buffer, pos - array->buffer->used +
-				   array->element_size);
-	} else {
-		buffer_write_zero(array->buffer, pos, array->element_size);
-	}
+	buffer_write_zero(array->buffer, idx * array->element_size,
+			  array->element_size);
 }
 
 void *array_insert_space_i(struct array *array, unsigned int idx)
@@ -69,6 +48,59 @@ bool array_cmp_i(const struct array *array1, const struct array *array2)
 	return buffer_cmp(array1->buffer, array2->buffer);
 }
 
+bool array_equal_fn_i(const struct array *array1, const struct array *array2,
+		      int (*cmp)(const void *, const void*))
+{
+	unsigned int count1, count2, i, size;
+
+	if (!array_is_created_i(array1) || array1->buffer->used == 0)
+		return !array_is_created_i(array2) || array2->buffer->used == 0;
+
+	if (!array_is_created_i(array2))
+		return FALSE;
+
+	count1 = array_count_i(array1); count2 = array_count_i(array2);
+	if (count1 != count2)
+		return FALSE;
+
+	size = array1->element_size;
+	i_assert(size == array2->element_size);
+
+	for (i = 0; i < count1; i++) {
+		if (cmp(CONST_PTR_OFFSET(array1->buffer->data, i * size),
+			CONST_PTR_OFFSET(array2->buffer->data, i * size)) != 0)
+			return FALSE;
+	}
+	return TRUE;
+}
+
+bool array_equal_fn_ctx_i(const struct array *array1, const struct array *array2,
+			  int (*cmp)(const void *, const void *, const void *),
+			  const void *context)
+{
+	unsigned int count1, count2, i, size;
+
+	if (!array_is_created_i(array1) || array1->buffer->used == 0)
+		return !array_is_created_i(array2) || array2->buffer->used == 0;
+
+	if (!array_is_created_i(array2))
+		return FALSE;
+
+	count1 = array_count_i(array1); count2 = array_count_i(array2);
+	if (count1 != count2)
+		return FALSE;
+
+	size = array1->element_size;
+	i_assert(size == array2->element_size);
+
+	for (i = 0; i < count1; i++) {
+		if (cmp(CONST_PTR_OFFSET(array1->buffer->data, i * size),
+			CONST_PTR_OFFSET(array2->buffer->data, i * size), context) != 0)
+			return FALSE;
+	}
+	return TRUE;
+}
+
 void array_reverse_i(struct array *array)
 {
 	const size_t element_size = array->element_size;
@@ -106,3 +138,19 @@ void *array_bsearch_i(struct array *array, const void *key,
 	return bsearch(key, array->buffer->data,
 		       count, array->element_size, cmp);
 }
+
+const void *array_lsearch_i(const struct array *array, const void *key,
+			    int (*cmp)(const void *, const void *))
+{
+	const void * const data = buffer_get_data(array->buffer, NULL);
+	const unsigned int s = array->element_size;
+	unsigned int idx;
+
+	for (idx = 0; idx < array_count_i(array); idx++) {
+		if (cmp(key, CONST_PTR_OFFSET(data, idx * s)) == 0) {
+			return PTR_OFFSET(data, idx * s);
+		}
+	}
+
+	return NULL;
+}
