@@ -6,6 +6,8 @@
 
 void test_aqueue(void);
 void test_array(void);
+enum fatal_test_state fatal_array(int);
+void test_base32(void);
 void test_base64(void);
 void test_bits(void);
 void test_bsearch_insert_pos(void);
@@ -44,6 +46,7 @@ void test_strfuncs(void);
 void test_strnum(void);
 void test_str_find(void);
 void test_str_sanitize(void);
+void test_str_table(void);
 void test_time_util(void);
 void test_unichar(void);
 void test_utc_mktime(void);
