@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2005-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -6,6 +6,8 @@
 #include "istream.h"
 #include "llist.h"
 #include "hash.h"
+#include "time-util.h"
+#include "process-title.h"
 #include "master-interface.h"
 #include "master-service.h"
 #include "log-error-buffer.h"
@@ -16,6 +18,13 @@
 #include <unistd.h>
 
 #define FATAL_QUEUE_TIMEOUT_MSECS 500
+#define MAX_MSECS_PER_CONNECTION 100
+
+/* Log a warning after 1 secs when we've been all the time busy writing the
+   log connection. */
+#define LOG_WARN_PENDING_COUNT (1000 / MAX_MSECS_PER_CONNECTION)
+/* If we keep beeing busy, log a warning every 60 seconds. */
+#define LOG_WARN_PENDING_INTERVAL (60 * LOG_WARN_PENDING_COUNT)
 
 struct log_client {
 	struct ip_addr ip;
@@ -35,15 +44,45 @@ struct log_connection {
 	char *default_prefix;
 	HASH_TABLE(void *, struct log_client *) clients;
 
+	unsigned int pending_count;
+
 	unsigned int master:1;
 	unsigned int handshaked:1;
 };
 
 static struct log_connection *log_connections = NULL;
 static ARRAY(struct log_connection *) logs_by_fd;
+static unsigned int global_pending_count;
+static struct log_connection *last_pending_log;
 
 static void log_connection_destroy(struct log_connection *log);
 
+static void log_refresh_proctitle(void)
+{
+	if (!verbose_proctitle)
+		return;
+
+	if (global_pending_count == 0)
+		process_title_set("");
+	else if (last_pending_log == NULL) {
+		process_title_set(t_strdup_printf(
+			"[%u services too fast]", global_pending_count));
+	} else if (global_pending_count > 1) {
+		process_title_set(t_strdup_printf(
+			"[%u services too fast, last: %d/%d/%s]",
+			global_pending_count,
+			last_pending_log->fd,
+			last_pending_log->listen_fd,
+			last_pending_log->default_prefix));
+	} else {
+		process_title_set(t_strdup_printf(
+			"[service too fast: %d/%d/%s]",
+			last_pending_log->fd,
+			last_pending_log->listen_fd,
+			last_pending_log->default_prefix));
+	}
+}
+
 static struct log_client *log_client_get(struct log_connection *log, pid_t pid)
 {
 	struct log_client *client;
@@ -143,26 +182,26 @@ log_parse_master_line(const char *line, const struct timeval *log_time,
 	struct log_client *client;
 	const char *p, *p2, *cmd;
 	unsigned int count;
-	int service_fd;
+	unsigned int service_fd;
 	pid_t pid;
 
 	p = strchr(line, ' ');
-	if (p == NULL || (p2 = strchr(++p, ' ')) == NULL) {
+	if (p == NULL || (p2 = strchr(++p, ' ')) == NULL ||
+	    str_to_uint(t_strcut(line, ' '), &service_fd) < 0) {
 		i_error("Received invalid input from master: %s", line);
 		return;
 	}
-	service_fd = atoi(t_strcut(line, ' '));
 	pid = strtol(t_strcut(p, ' '), NULL, 10);
 	cmd = p2 + 1;
 
 	logs = array_get(&logs_by_fd, &count);
-	if (service_fd >= (int)count || logs[service_fd] == NULL) {
-		if (strcmp(cmd, "BYE") == 0 && service_fd < (int)count) {
+	if (service_fd >= count || logs[service_fd] == NULL) {
+		if (strcmp(cmd, "BYE") == 0 && service_fd < count) {
 			/* master is probably shutting down and we already
 			   noticed the log fd closing */
 			return;
 		}
-		i_error("Received master input for invalid service_fd %d: %s",
+		i_error("Received master input for invalid service_fd %u: %s",
 			service_fd, line);
 		return;
 	}
@@ -243,7 +282,7 @@ static int log_connection_handshake(struct log_connection *log)
 
 	ret = i_stream_read(log->input);
 	if (ret < 0) {
-		i_error("read(log pipe) failed: %m");
+		i_error("read(log %s) failed: %m", log->default_prefix);
 		return -1;
 	}
 	if ((size_t)ret < sizeof(handshake)) {
@@ -264,6 +303,7 @@ static int log_connection_handshake(struct log_connection *log)
 		i_error("Missing prefix data in handshake");
 		return -1;
 	}
+	i_free(log->default_prefix);
 	log->default_prefix = i_strndup(data + sizeof(handshake),
 					handshake.prefix_len);
 	i_stream_skip(log->input, sizeof(handshake) + handshake.prefix_len);
@@ -284,7 +324,7 @@ static void log_connection_input(struct log_connection *log)
 {
 	const char *line;
 	ssize_t ret;
-	struct timeval now;
+	struct timeval now, start_timeval;
 	struct tm tm;
 
 	if (!log->handshaked) {
@@ -294,23 +334,48 @@ static void log_connection_input(struct log_connection *log)
 		}
 	}
 
+	io_loop_time_refresh();
+	start_timeval = ioloop_timeval;
 	while ((ret = i_stream_read(log->input)) > 0 || ret == -2) {
 		/* get new timestamps for every read() */
-		if (gettimeofday(&now, NULL) < 0)
-			i_panic("gettimeofday() failed: %m");
+		now = ioloop_timeval;
 		tm = *localtime(&now.tv_sec);
 
 		while ((line = i_stream_next_line(log->input)) != NULL)
 			log_it(log, line, &now, &tm);
+		io_loop_time_refresh();
+		if (timeval_diff_msecs(&ioloop_timeval, &start_timeval) > MAX_MSECS_PER_CONNECTION)
+			break;
 	}
 
 	if (log->input->eof)
 		log_connection_destroy(log);
 	else if (log->input->stream_errno != 0) {
-		i_error("read(log pipe) failed: %m");
+		i_error("read(log %s) failed: %m", log->default_prefix);
 		log_connection_destroy(log);
 	} else {
 		i_assert(!log->input->closed);
+		if (ret == 0) {
+			if (log->pending_count > 0) {
+				log->pending_count = 0;
+				i_assert(global_pending_count > 0);
+				global_pending_count--;
+				if (log == last_pending_log)
+					last_pending_log = NULL;
+				log_refresh_proctitle();
+			}
+			return;
+		}
+		last_pending_log = log;
+		if (log->pending_count++ == 0) {
+			global_pending_count++;
+			log_refresh_proctitle();
+		}
+		if (log->pending_count == LOG_WARN_PENDING_COUNT ||
+		    (log->pending_count % LOG_WARN_PENDING_INTERVAL) == 0) {
+			i_warning("Log connection fd %d listen_fd %d prefix '%s' is sending input faster than we can write",
+				  log->fd, log->listen_fd, log->default_prefix);
+		}
 	}
 }
 
@@ -325,6 +390,7 @@ void log_connection_create(struct log_error_buffer *errorbuf,
 	log->listen_fd = listen_fd;
 	log->io = io_add(fd, IO_READ, log_connection_input, log);
 	log->input = i_stream_create_fd(fd, PIPE_BUF, FALSE);
+	log->default_prefix = i_strdup_printf("listen_fd %d", listen_fd);
 	hash_table_create_direct(&log->clients, default_pool, 0);
 	array_idx_set(&logs_by_fd, listen_fd, &log);
 
