@@ -1,7 +1,14 @@
 #ifndef LOGIN_PROXY_H
 #define LOGIN_PROXY_H
 
-#include "network.h"
+#include "net.h"
+
+/* Max. number of embedded proxying connections until proxying fails.
+   This is intended to avoid an accidental configuration where two proxies
+   keep connecting to each others, both thinking the other one is supposed to
+   handle the user. This only works if both proxies support the Dovecot
+   TTL extension feature. */
+#define LOGIN_PROXY_TTL 5
 
 struct client;
 struct login_proxy;
@@ -17,8 +24,7 @@ enum login_proxy_ssl_flags {
 
 struct login_proxy_settings {
 	const char *host;
-	struct ip_addr ip;
-	const char *dns_client_socket_path;
+	struct ip_addr ip, source_ip;
 	unsigned int port;
 	unsigned int connect_timeout_msecs;
 	/* send a notification about proxy connection to proxy-notify pipe
