@@ -1,4 +1,4 @@
-/* Copyright (c) 2011-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2011-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "mail-namespace.h"
@@ -25,7 +25,8 @@ fts_namespace_find(struct mail_user *user, const char *ns_prefix,
 	}
 
 	if (fts_list_backend(ns->list) == NULL) {
-		i_error("fts not enabled for user's namespace %s", ns_prefix);
+		i_error("fts not enabled for user's namespace %s",
+			ns_prefix != NULL ? ns_prefix : "INBOX");
 		return -1;
 	}
 	*ns_r = ns;
