@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2005-2015 Dovecot authors, see the included COPYING file */
 
 /* Only for reporting filesystem quota */
 
@@ -113,6 +113,8 @@ static int fs_quota_init(struct quota_root *_root, const char *args,
 			root->inode_per_mail = TRUE;
 		else if (strcmp(*tmp, "noenforcing") == 0)
 			_root->no_enforcing = TRUE;
+		else if (strcmp(*tmp, "hidden") == 0)
+			_root->hidden = TRUE;
 		else if (strncmp(*tmp, "mount=", 6) == 0) {
 			i_free(root->storage_mount_path);
 			root->storage_mount_path = i_strdup(*tmp + 6);
