@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2014 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2015 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 
@@ -226,7 +226,7 @@ static void proxy_open_logs(struct rawlog_proxy *proxy, const char *path,
 	if (ip_addr != NULL &&
 	    (proxy->flags & RAWLOG_FLAG_LOG_IP_IN_FILENAME) != 0)
 		str_printfa(path_prefix, "%s-", ip_addr);
-	str_printfa(path_prefix, "%s-%s.", timestamp, dec2str(getpid()));
+	str_printfa(path_prefix, "%s-%s", timestamp, dec2str(getpid()));
 
 	if ((proxy->flags & RAWLOG_FLAG_LOG_INPUT) != 0) {
 		fname = t_strdup_printf("%s.in", str_c(path_prefix));
@@ -362,7 +362,7 @@ int main(int argc, char *argv[])
 	int c;
 
 	master_service = master_service_init("rawlog", 0,
-					     &argc, &argv, "+f:obit");
+					     &argc, &argv, "+f:obIt");
 	while ((c = master_getopt(master_service)) > 0) {
 		switch (c) {
 		case 'f':
@@ -376,7 +376,7 @@ int main(int argc, char *argv[])
 		case 'b':
 			flags |= RAWLOG_FLAG_LOG_BOUNDARIES;
 			break;
-		case 'i':
+		case 'I':
 			flags |= RAWLOG_FLAG_LOG_IP_IN_FILENAME;
 			break;
 		case 't':
@@ -390,7 +390,7 @@ int main(int argc, char *argv[])
 	argv += optind;
 
 	if (argc < 1)
-		i_fatal("Usage: rawlog [-f in|out] [-i] [-b] [-t] <binary> <arguments>");
+		i_fatal("Usage: rawlog [-f in|out] [-I] [-b] [-t] <binary> <arguments>");
 
 	master_service_init_log(master_service, "rawlog: ");
 	master_service_init_finish(master_service);
